@@ -256,8 +256,9 @@ smbc_free_context(SMBCCTX *context,
 
                 f = context->internal->files;
                 while (f) {
+			SMBCFILE *next = f->next;
                         smbc_getFunctionClose(context)(context, f);
-                        f = f->next;
+			f = next;
                 }
                 context->internal->files = NULL;
 
@@ -485,7 +486,7 @@ smbc_option_get(SMBCCTX *context,
 
                 for (s = context->internal->servers; s; s = s->next) {
                         num_servers++;
-                        if (!cli_state_is_encryption_on(s->cli)) {
+                        if (!smb1cli_conn_encryption_on(s->cli->conn)) {
                                 return (void *)false;
                         }
                 }
