@@ -219,7 +219,7 @@ bool is_posix_locked(files_struct *fsp,
 			enum brl_flavour lock_flav);
 bool posix_locking_init(bool read_only);
 bool posix_locking_end(void);
-int fd_close_posix(struct files_struct *fsp);
+int fd_close_posix(const struct files_struct *fsp);
 bool set_posix_lock_windows_flavour(files_struct *fsp,
 			uint64_t u_offset,
 			uint64_t u_count,
@@ -239,6 +239,7 @@ bool set_posix_lock_posix_flavour(files_struct *fsp,
 			uint64_t u_offset,
 			uint64_t u_count,
 			enum brl_type lock_type,
+			const struct lock_context *lock_ctx,
 			int *errno_ret);
 bool release_posix_lock_posix_flavour(files_struct *fsp,
 				uint64_t u_offset,
